@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 
 #
-# Copyright (C) 2022 Jerry Hoogenboom
+# Copyright (C) 2024 Jerry Hoogenboom
 #
 # This file is part of FDSTools, data analysis tools for Massively
 # Parallel Sequencing of forensic DNA markers.
@@ -31,14 +31,14 @@ from strnaming import classes, libstrnaming, libsequence
 from .seq import PAT_SEQ_RAW, PAT_SEQ_IUPAC
 
 # Patterns that match (parts of) an STR definition.
-PAT_STR_DEF = re.compile("^(?:(?:(?<=^)|(?<!^)\s+)[ACGT]+\s+\d+\s+\d+)*$")
-PAT_STR_DEF_BLOCK = re.compile("([ACGT]+)\s+(\d+)\s+(\d+)")
+PAT_STR_DEF = re.compile(r"^(?:(?:(?<=^)|(?<!^)\s+)[ACGT]+\s+\d+\s+\d+)*$")
+PAT_STR_DEF_BLOCK = re.compile(r"([ACGT]+)\s+(\d+)\s+(\d+)")
 
 # Pattern that matches a chromosome name/number.
-PAT_CHROMOSOME = re.compile("^(?:[Cc][Hh][Rr](?:[Oo][Mm])?)?([1-9XYM]|1\d|2[0-2])$")
+PAT_CHROMOSOME = re.compile(r"^(?:[Cc][Hh][Rr](?:[Oo][Mm])?)?([1-9XYM]|1\d|2[0-2])$")
 
 # Pattern to split a comma-, tab- or space-separated list.
-PAT_SPLIT = re.compile("\s*[, \t]\s*")
+PAT_SPLIT = re.compile(r"\s*[, \t]\s*")
 
 # TextWrapper object for formatting help texts in generated INI files.
 INI_COMMENT = textwrap.TextWrapper(width=79, initial_indent="; ",
@@ -81,12 +81,32 @@ def add_legacy_range(reported_range_store, marker, prefix, suffix, blocks, optio
         raise ValueError(
             "Gapped or combined genomic range is not supported for STR marker %s" % marker)
 
+    # Make sure that the [flanks] and [flank_length] values are compatible with each other.
+    if not options["autoload_reference"]:
+        if "flank_length" in options:
+            if not all(options["flanks"]):
+                raise ValueError(
+                    "It is not possible to specify flank_length without also explicitly "
+                    "specifying the flanking sequence for explicitly-defined marker %s" % marker)
+            for side, given_length, max_length in zip(
+                    ("left", "right"), map(len, options["flanks"]), options["flank_length"][1::2]):
+                if max_length > given_length:
+                    raise ValueError(
+                        "Maximum %s flank length of %i was specified for explicitly-defined "
+                        "marker %s, but only %i flanking bases were provided" %
+                        (side, max_length, marker, given_length))
+        else:
+            # Since autoload_reference is False, we must set a limit!
+            options["flank_length"] = [
+                1, len(options["flanks"][0]),
+                1, len(options["flanks"][1])]
+
     # Find units and overlong_gap.
     overlong_gap = ""
     units = {}
     for unit, min_repeats, max_repeats in blocks:
         if max_repeats == 1:
-            if len(unit) > max(libstrnaming.NAMING_OPTIONS["max_gap"], len(overlong_gap)):
+            if len(unit) > max(libstrnaming.MAX_GAP_LENGTH, len(overlong_gap)):
                 overlong_gap = unit
         else:
             units[unit] = units.get(unit, 0) + max_repeats
@@ -131,7 +151,7 @@ def add_legacy_range(reported_range_store, marker, prefix, suffix, blocks, optio
 
     if stretches:
         # Make sure the structure is long enough to be accepted by STRNaming.
-        while end - stretches[0][0] < libstrnaming.NAMING_OPTIONS["min_structure_length"]:
+        while end - stretches[0][0] < libstrnaming.MIN_STRUCTURE_LENGTH:
             stretches[-1][1] += len(stretches[-1][2])
             end += len(stretches[-1][2])
             refseq[-1] += stretches[-1][2]
@@ -168,7 +188,7 @@ def parse_library(handle):
     markers = {}
     ini = RawConfigParser()
     ini.optionxform = str
-    ini.readfp(handle)
+    ini.read_file(handle)
     for section in ini.sections():
         for marker in ini.options(section):
             value = ini.get(section, marker).split(";", 1)[0].strip()
@@ -177,17 +197,37 @@ def parse_library(handle):
                 value = PAT_SPLIT.split(value)
                 if len(value) != 2:
                     raise ValueError(
-                        "For marker %s, %i flanking sequences were given, "
-                        "need exactly 2" % (marker, len(value)))
+                        "For marker %s, %i flanking sequences were given, need exactly two "
+                        "(note: specify 'REF' to automatically load genome reference)"
+                        % (marker, len(value)))
                 for i, val in enumerate(value):
-                    if PAT_SEQ_IUPAC.match(val) is None:
-                        try:
-                            value[i] = int(val)
-                            if value[i] < 1:
-                                raise ValueError
-                        except:
-                            raise ValueError(
-                                "Flanking sequence '%s' of marker %s is invalid" % (val, marker))
+                    if val == "REF":
+                        value[i] = ""
+                    elif val.isdigit():
+                        raise ValueError(
+                            "Specifying a flank length (%s for marker %s) in the [flanks] "
+                            "section of the library file is no longer supported. "
+                            "Use the [flank_length] section to specify marker-specific flank "
+                            "length limitations in the library file; specify 'REF' in the "
+                            "[flanks] section to automatically load genome reference"
+                            % (val, marker))
+                    elif PAT_SEQ_IUPAC.match(val) is None:
+                        raise ValueError(
+                            "Flanking sequence '%s' of marker %s is not a valid sequence "
+                            "(note: specify 'REF' to automatically load genome reference)"
+                            % (val, marker))
+            elif section_low == "flank_length":
+                value = PAT_SPLIT.split(value)
+                if len(value) != 4:
+                    raise ValueError(
+                        "Flank length should be four numbers, but %i values were "
+                        "given for marker %s" % (len(value), marker))
+                for i, val in enumerate(value):
+                    if not val.isdigit() or not int(val):
+                        raise ValueError(
+                            "Flank length '%s' of marker %s is not a valid positive "
+                            "integer" % (val, marker))
+                    value[i] = int(val)
             elif section_low in ("prefix", "suffix"):
                 if not PAT_SEQ_RAW.match(value):
                     raise ValueError(
@@ -206,7 +246,7 @@ def parse_library(handle):
                         raise ValueError(
                             "Position '%s' of marker %s is not a valid integer"
                             % (value[i], marker))
-                    if not i % 2 and pos[-2] >= pos[-1]:
+                    if not i % 2 and pos[-2] > pos[-1]:
                         raise ValueError(
                             "End position %i of marker %s must be higher than "
                             "corresponding start position %i" % (pos[-1], marker, pos[-2]))
@@ -284,6 +324,9 @@ def parse_library(handle):
         "explicit STR": ("prefix", "suffix", "repeat", "length_adjust", "block_length"),
         "explicit non-STR": ("no_repeat",),
         "STRNaming-specific": tuple()}  # NOTE: No STRNaming-specific settings now...
+
+    # Get a list of ranges, so that we can load the STR structures in one call.
+    ranges = {}
     for marker, settings in markers.items():
         groups = [mutex_group for mutex_group, sections in MUTEX_GROUPS.items()
             if any(section in settings for section in sections)]
@@ -293,17 +336,40 @@ def parse_library(handle):
                     "%s sections (%s)" % (group, ", ".join(
                         section for section in MUTEX_GROUPS[group] if section in settings))
                     for group in groups)))
-        options = {option: settings[option] for option in {"flanks", "max_expected_copies",
+        if "explicit STR" not in groups and "explicit non-STR" not in groups:
+            try:
+                genome_position = settings["genome_position"]
+            except KeyError:
+                raise ValueError("No genome_position or explicit repeat or no_repeat "
+                                 "configuration provided for marker %s" % marker)
+            if not len(genome_position) % 2:
+                raise ValueError(
+                    "Invalid genomic position given for marker %s: need an odd number of values "
+                    "(chromosome, start position, end position[, start2, end2, ...])" % marker)
+            if len(genome_position) == 3 and "microhaplotype_positions" not in settings:
+                chromosome, start, end = genome_position
+                if chromosome not in ranges:
+                    ranges[chromosome] = [(start, end + 1)]
+                else:
+                    ranges[chromosome].append((start, end + 1))
+
+    # Load the STR structures on the given ranges.
+    structure_store = reported_range_store.get_structure_store()
+    for chromosome, ranges_here in ranges.items():
+        structure_store.load_within_ranges(chromosome, sorted(ranges_here))
+
+    # Now add all markers to the ReportedRangeStore.
+    for marker, settings in markers.items():
+        groups = [mutex_group for mutex_group, sections in MUTEX_GROUPS.items()
+            if any(section in settings for section in sections)]
+        options = {option: settings[option] for option in {
+                "flanks", "flank_length", "max_expected_copies",
                 "expected_allele_length", "microhaplotype_positions"} & settings.keys()}
         if "explicit STR" in groups:
             # Legacy FDSTools-style definition of an STR marker.
-            # TODO: Alias of STR markers was defined as excluding the prefix/suffix!
             if "flanks" not in options:
                 options["flanks"] = ("", "")
-            elif any(isinstance(flank, int) for flank in options["flanks"]):
-                raise ValueError(
-                    "Please specify an explit flanking sequence, not just a length, for marker %s"
-                        % marker)
+            options["autoload_reference"] = False
             reported_range = add_legacy_range(reported_range_store, marker,
                 settings.get("prefix", ""),
                 settings.get("suffix", ""),
@@ -319,10 +385,6 @@ def parse_library(handle):
             # Legacy FDSTools-style definition of a non-STR marker.
             if "flanks" not in options:
                 options["flanks"] = ("", "")
-            elif any(isinstance(flank, int) for flank in options["flanks"]):
-                raise ValueError(
-                    "Please specify an explit flanking sequence, not just a length, for marker %s"
-                        % marker)
             refseq = settings["no_repeat"]
             pos = None
             if "genome_position" in settings:
@@ -334,30 +396,22 @@ def parse_library(handle):
                     raise ValueError(
                         "Length of reference sequence of marker %s is %i bases, but "
                         "genome positions add up to %i bases" % (marker, len(refseq), length))
+            options["autoload_reference"] = (len(pos or []) % 2) != 0  # If end pos given.
             add_legacy_range(reported_range_store, marker, refseq, "", [], options, pos)
         else:
             # Use STRNaming for this marker.
-            try:
-                genome_position = settings["genome_position"]
-            except KeyError:
-                raise ValueError("No genome_position or explicit repeat or no_repeat "
-                                 "configuration provided for marker %s" % marker)
-            if not len(genome_position) % 2:
-                raise ValueError(
-                    "Invalid genomic position given for marker %s: need an odd number of values "
-                    "(chromosome, start position, end position[, start2, end2, ...])" % marker)
-            # TODO: Alias of STR markers was defined as excluding the prefix/suffix!
-            if len(genome_position) == 3:
+            genome_position = settings["genome_position"]
+            if len(genome_position) == 3 and "microhaplotype_positions" not in settings:
                 chromosome, start, end = genome_position
                 reported_range_store.add_range(
-                    marker, chromosome, start, end + 1, load_structures=True, options=options)
+                    marker, chromosome, start, end + 1, options=options)
             else:
                 reported_range_store.add_complex_range(marker, genome_position, options=options)
 
         if "microhaplotype_positions" in settings:
             # Put Ns in reporting range refseq for microhaplotype markers.
             reported_range = reported_range_store.get_range(marker)
-            if reported_range.library:
+            if getattr(reported_range, "library", False):
                 raise ValueError(
                     "Cannot define microhaplotype positions for STR marker %s" % marker)
             refseq = list(reported_range.refseq)
